"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunPath = void 0;
const Util_1 = require("./Util");
const consts_1 = require("./consts");
const Functions_1 = require("./evaluate/Functions");
const Processor_1 = require("./evaluate/Processor");
const Variables_1 = require("./evaluate/Variables");
const Json_1 = require("./nodes/Json");
const Nodes_1 = require("./nodes/Nodes");
const Postfix_1 = require("./path/Postfix");
class RunPath {
    constructor({ json, functions, variables, outputOptions }) {
        this.options = {};
        this.caller = 'main';
        this.nodesInstance = new Nodes_1.Nodes();
        this.functionInstance = new Functions_1.Functions(functions);
        this.postfixInstance = new Postfix_1.Postfix(this.caller, this.functionInstance);
        this.nodes = this.nodesInstance.jsonToNodes(json, this.caller);
        this.variablesInstance = new Variables_1.Variables({ root: json, nodes: this.nodes }, variables);
        this.processorInstance = new Processor_1.Processor(this.nodesInstance, this.variablesInstance, this.functionInstance);
        this.toJsonInstance = new Json_1.Json();
        this.variablesInstance.setInstances({
            postfixInstance: this.postfixInstance,
            processorInstance: this.processorInstance,
            toJsonInstance: this.toJsonInstance
        });
        this.options.nodes = (outputOptions === null || outputOptions === void 0 ? void 0 : outputOptions.nodes) || false;
    }
    callbackMode({ path, then, description }) {
        let error;
        let value = [];
        let nodesValue;
        try {
            const postfixPath = this.postfixInstance.toPostfix(path);
            this.processorInstance.processPostfixPath(postfixPath);
            const item = this.processorInstance.output.pop();
            if (item) {
                value = item.type !== consts_1.TYPES.nodes
                    ? item.value
                    : this.toJsonInstance.reconstruct(item.value);
            }
            if (item && item.type === consts_1.TYPES.nodes && this.options.nodes) {
                nodesValue = item.value;
            }
        }
        catch (e) {
            error = `${e.message}${description ? ` for the scenario:"${description}"` : ` for path "${path}"`}`;
        }
        then({
            path,
            description,
            value,
            error,
            nodes: this.options.nodes ? Util_1.nodesOps.get.nodeList(true) : undefined,
            nodesValue
        });
    }
    stringPathMode(path) {
        let value = [];
        try {
            const postfixPath = this.postfixInstance.toPostfix(path);
            this.processorInstance.processPostfixPath(postfixPath);
            const item = this.processorInstance.output.pop();
            if (item) {
                value = item.type !== consts_1.TYPES.nodes ? item.value : this.toJsonInstance.reconstruct(item.value);
            }
        }
        catch (e) {
            throw new Error(`${e.message} for path "${path}"`);
        }
        return value;
    }
    reset() {
        this.processorInstance.reset();
    }
}
exports.RunPath = RunPath;
//# sourceMappingURL=RunPath.js.map