"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.throwError = exports.opFunc = exports.nodesOps = void 0;
const consts_1 = require("./consts");
const State_1 = require("./nodes/State");
const consts_2 = require("./nodes/consts");
const Json_1 = require("./nodes/Json");
const nsInstance = State_1.NodesState.getInstance();
exports.nodesOps = {
    tests: {
        exists: (id) => {
            return !!nsInstance.getNodeById(id);
        },
        isEqual: (node1, node2) => {
            if (node1[consts_2.KEYS.valueType] !== node2[consts_2.KEYS.valueType]) {
                return false;
            }
            if (node1[consts_2.KEYS.value] === consts_2.SYMBOLS.object) {
                return exports.nodesOps.tests.equal.object(node1, node2);
            }
            else if (node1[consts_2.KEYS.value] === consts_2.SYMBOLS.array) {
                return exports.nodesOps.tests.equal.array(node1, node2);
            }
            return node1[consts_2.KEYS.value] === node2[consts_2.KEYS.value];
        },
        equal: {
            object: (node1, node2) => {
                const node1Children = exports.nodesOps.get.children(node1);
                const node2Children = exports.nodesOps.get.children(node2);
                if (node1Children.length === node2Children.length) {
                    return node1Children.every(n1 => {
                        return node2Children.some(n2 => {
                            return n1[consts_2.KEYS.name] === n2[consts_2.KEYS.name] &&
                                n1[consts_2.KEYS.value] === n2[consts_2.KEYS.value] &&
                                (![consts_2.SYMBOLS.array, consts_2.SYMBOLS.object].includes(n1[consts_2.KEYS.value]) ||
                                    exports.nodesOps.tests.equal[consts_2.KEYS.valueType](n1, n2));
                        });
                    });
                }
                return false;
            },
            array: (node1, node2) => {
                const node1Children = exports.nodesOps.get.children(node1);
                const node2Children = exports.nodesOps.get.children(node2);
                if (node1Children.length === node2Children.length) {
                    return node1Children.every(n1 => {
                        return node2Children.some(n2 => {
                            return n1[consts_2.KEYS.name] === n2[consts_2.KEYS.name] &&
                                n1[consts_2.KEYS.arrayPosition] === n2[consts_2.KEYS.arrayPosition] &&
                                n1[consts_2.KEYS.value] === n2[consts_2.KEYS.value] &&
                                (![consts_2.SYMBOLS.array, consts_2.SYMBOLS.object, consts_2.SYMBOLS.ao].includes(n1[consts_2.KEYS.value]) ||
                                    exports.nodesOps.tests.equal[n1[consts_2.KEYS.valueType]](n1, n2));
                        });
                    });
                }
                return false;
            }
        }
    },
    get: {
        root: (nodes) => {
            if (nodes.length) {
                if (nodes[0][consts_2.KEYS.name] === consts_2.SYMBOLS.root) {
                    return [nodes[0]];
                }
                else {
                    return exports.nodesOps.get.byName(nodes, consts_2.SYMBOLS.root)[0];
                }
            }
        },
        byName: (nodes, name) => {
            if (!name || name === '*')
                return nodes;
            return nodes.filter((c) => {
                return c[consts_2.KEYS.name] === name;
            });
        },
        byGroup: (nodes, group) => {
            return nodes.filter(c => {
                return c[consts_2.KEYS.group] === group;
            });
        },
        byParentId: (nodes, parentId) => {
            return nodes.filter(c => {
                return c[consts_2.KEYS.links].parentId === parentId;
            });
        },
        byDepth: (nodes, depth) => {
            return nodes.filter(c => {
                return c[consts_2.KEYS.depth] === depth;
            });
        },
        descendants: (current, nodeName) => {
            return current
                ? exports.nodesOps.get.byName(nsInstance.getNodesByIds(current[consts_2.KEYS.links].descendantIds), nodeName)
                : [];
        },
        ancestors: (current, nodeName) => {
            return current
                ? exports.nodesOps.get.byName(nsInstance.getNodesByIds(current[consts_2.KEYS.links].ancestorIds), nodeName)
                : [];
        },
        siblings: (current, nodeName) => {
            return current
                ? exports.nodesOps.get.byName(nsInstance.getNodesByIds(current[consts_2.KEYS.links].siblings), nodeName)
                : [];
        },
        children: (current, nodeName) => {
            return current
                ? exports.nodesOps.get.byName(nsInstance.getNodesByIds(current[consts_2.KEYS.links].childrenIds), nodeName)
                : [];
        },
        parent: (current, nodeName) => {
            if (!current)
                return [];
            let parent = nsInstance.getNodeById(current[consts_2.KEYS.links].parentId);
            if (parent[consts_2.KEYS.name] === consts_2.SYMBOLS.ao) {
                parent = nsInstance.getNodeById(parent[consts_2.KEYS.links].parentId);
            }
            return parent && ((!nodeName || nodeName === '*') || nodeName === parent[consts_2.KEYS.name]) ? parent : undefined;
        },
        nodeList: (byId) => {
            return nsInstance.getNodeList(byId);
        }
    },
    reconstruct: (nodes) => {
        const jsonInstance = new Json_1.Json();
        return jsonInstance.reconstruct(nodes);
    }
};
exports.opFunc = {
    /**
     *
     * @param context the caller name
     * @param expectedTypes array of string of allowable types
     * @param item the interrogated stack item
     * @returns the actual value
     */
    getValueByType: (context, expectedTypes, item) => {
        if (!expectedTypes.length) {
            (0, exports.throwError)(context, 'Missing type argument.');
        }
        if (!item) {
            (0, exports.throwError)(context, `Missing stack item/argument`);
        }
        if (![...expectedTypes, consts_1.TYPES.nodes].includes(item.type)) {
            (0, exports.throwError)(context, `Invalid type "${item.type}" - was expecting "${expectedTypes.join(',')}" type.`);
        }
        if (item.type === consts_1.TYPES.nodes) {
            if (!item.value.length) {
                (0, exports.throwError)(context, `Was expecting at least one node value.`);
            }
            if (!expectedTypes.includes(item.value[0][consts_2.KEYS.valueType])) {
                (0, exports.throwError)(context, `Invalid value type "${item.value[0][consts_2.KEYS.valueType]}" - was expecting "${expectedTypes.join(',')}" value type.`);
            }
            return item.value[0][consts_2.KEYS.value];
        }
        else { // number
            return item.value;
        }
    },
    validate: {
        exists: (item, context) => {
            if (!item) {
                (0, exports.throwError)(context, `No item exists`);
            }
            if (item.type === consts_1.TYPES.nodes && !item.value.length) {
                (0, exports.throwError)(context, `No node items exists`);
            }
        },
        /**
         * Validate the length of the number of nodes in the value list
         * If invalid, throws an error
         *
         * @param expectedLength
         *  the length of the nodes' value expected
         * @param item
         *  the stack item type in the format {type, value}
         * @param symbol?
         *  optional symbol used to test against (>=, >, <=, <, =).
         *  If not defined it is tested against the expectedLength
         * @param context?
         *  optional context (original caller)
         * @returns
         *  boolean
         */
        numberOfNodeItems: (item, expectedLength, symbol, context) => {
            context = context ? `${context}.validate.numberOfNodeItems` : 'validate.numberOfNodeItems';
            if (item.type !== consts_1.TYPES.nodes) {
                (0, exports.throwError)(context, `Invalid type "${item.type}" - expecting "nodes".`);
            }
            switch (symbol) {
                case '>':
                    if (item.value.length <= expectedLength) {
                        (0, exports.throwError)(context, `Invalid number of elements in value. Expecting greater than ${expectedLength} elements but received ${item.value.length}.`);
                    }
                    break;
                case '>=':
                    if (item.value.length < expectedLength) {
                        (0, exports.throwError)(context, `Invalid number of elements in value. Expecting greater than or equal to ${expectedLength} elements but received ${item.value.length}.`);
                    }
                    break;
                case '<':
                    if (item.value.length >= expectedLength) {
                        (0, exports.throwError)(context, `Invalid number of elements in value. Expecting less than ${expectedLength} elements but received ${item.value.length}.`);
                    }
                    break;
                case '<=':
                    if (item.value.length > expectedLength) {
                        (0, exports.throwError)(context, `Invalid number of elements in value. Expecting less than or equal to ${expectedLength} elements but received ${item.value.length}.`);
                    }
                    break;
                default:
                    if (expectedLength !== item.value.length) {
                        (0, exports.throwError)(context, `Invalid number of elements in value. Expecting ${expectedLength} elements but received ${item.value.length}.`);
                    }
            }
            return true;
        },
        areNumbers: (item, context) => {
            if (![consts_1.TYPES.number, consts_1.TYPES.nodes].includes(item.type)) {
                (0, exports.throwError)(context, `Invalid type "${item.type}" - was expecting "number" type.`);
            }
            if (item.type === consts_1.TYPES.nodes) {
                if (!item.value.every(node => node[consts_2.KEYS.valueType] === consts_1.TYPES.number)) {
                    (0, exports.throwError)(context, `Not all node elements are number types`);
                }
            }
        }
    }
};
const throwError = (context, msg) => {
    throw new Error(`Error thrown in ${context}: ${msg}`);
};
exports.throwError = throwError;
//# sourceMappingURL=Util.js.map