"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Functions = void 0;
const consts_1 = require("../nodes/consts");
const Util_1 = require("../Util");
const consts_2 = require("../consts");
class Functions {
    mergeCustomFunction(customFunctions) {
        if (!customFunctions) {
            this.mergedF = this.f;
            return;
        }
        this.mergedF = Object.assign(Object.assign({}, this.f), customFunctions);
    }
    ;
    getContext(operator) {
        return `Functions[${operator}]`;
    }
    ;
    constructor(customFunctions) {
        this.contextProperties = {
            lastIndex: null,
            firstIndex: null,
            index: null,
            current: null,
            isLast: false,
            isFirst: false,
            isFilterMode: false
        };
        this.f = {
            abs: (arg) => {
                const value = Util_1.opFunc.getValueByType(this.getContext('abs'), [consts_2.TYPES.number], arg);
                return {
                    type: consts_2.TYPES.number,
                    value: Math.abs(value)
                };
            },
            boolean: (arg) => {
                return {
                    type: consts_2.TYPES.boolean,
                    value: arg.type === consts_2.TYPES.nodes ? !!arg.value.length : !!arg.value
                };
            },
            ceiling: (arg) => {
                const value = Util_1.opFunc.getValueByType(this.getContext('ceiling'), [consts_2.TYPES.number], arg);
                return {
                    type: consts_2.TYPES.number,
                    value: Math.ceil(value)
                };
            },
            choose: (testStackItem, trueItem, falseItem) => {
                return (testStackItem.type === consts_2.TYPES.nodes && testStackItem.value.length || !!testStackItem.value) ? trueItem : falseItem;
            },
            concat: (...args) => {
                return {
                    type: consts_2.TYPES.string,
                    value: args.map(a => {
                        return this.f['string'](a).value;
                    }).join('')
                };
            },
            contains: (search, searchFor) => {
                const searchString = Util_1.opFunc.getValueByType(this.getContext('substring-before'), ['string'], search);
                const searchForString = Util_1.opFunc.getValueByType(this.getContext('substring-before'), ['string'], searchFor);
                return {
                    type: consts_2.TYPES.boolean,
                    value: searchString.includes(searchForString)
                };
            },
            count: (item) => {
                return {
                    type: consts_2.TYPES.number,
                    value: item ? (item.type === consts_2.TYPES.nodes ? item.value.length : 1) : 0
                };
            },
            false: () => {
                return {
                    type: consts_2.TYPES.boolean,
                    value: false
                };
            },
            first: () => {
                return {
                    type: consts_2.TYPES.number,
                    value: this.contextProperties.firstIndex
                };
            },
            floor: (arg) => {
                const value = Util_1.opFunc.getValueByType(this.getContext('floor'), [consts_2.TYPES.number], arg);
                return {
                    type: consts_2.TYPES.number,
                    value: Math.floor(value)
                };
            },
            last: () => {
                return {
                    type: consts_2.TYPES.number,
                    value: this.contextProperties.lastIndex
                };
            },
            'local-name': (item) => {
                return this.f.name(item);
            },
            // for now name and local-name is identical
            name: (item) => {
                let value = '';
                if (item && item.type === consts_2.TYPES.nodes && item.value.length) {
                    if (item.value[0][consts_1.KEYS.name] !== consts_1.SYMBOLS.root) {
                        value = item.value[0][consts_1.KEYS.name];
                    }
                    ;
                }
                else if (this.contextProperties.current) {
                    value = this.contextProperties.current[consts_1.KEYS.name];
                }
                return {
                    type: consts_2.TYPES.string,
                    value
                };
            },
            not: (item) => {
                Util_1.opFunc.validate.exists(item, this.getContext('not'));
                return {
                    type: consts_2.TYPES.boolean,
                    value: !this.f.boolean(item).value
                };
            },
            number: (arg) => {
                const value = Util_1.opFunc.getValueByType(this.getContext('number'), [consts_2.TYPES.number, consts_2.TYPES.string, consts_2.TYPES.boolean], arg);
                return {
                    type: consts_2.TYPES.number,
                    value: Number(value)
                };
            },
            round: (arg) => {
                const value = Util_1.opFunc.getValueByType(this.getContext('round'), [consts_2.TYPES.number], arg);
                return {
                    type: consts_2.TYPES.number,
                    value: Math.round(value)
                };
            },
            // converts the number or node into a string
            string: (arg) => {
                return {
                    type: consts_2.TYPES.string,
                    value: String(Util_1.opFunc.getValueByType(this.getContext('string'), [consts_2.TYPES.string, consts_2.TYPES.number, consts_2.TYPES.boolean], arg))
                };
            },
            'substring-before': (search, searchFor) => {
                const searchString = Util_1.opFunc.getValueByType(this.getContext('substring-before') + '(first argument)', ['string'], search);
                const searchForString = Util_1.opFunc.getValueByType(this.getContext('substring-before') + '(second argument)', ['string'], searchFor);
                const startIndex = searchString.indexOf(searchForString);
                return {
                    type: consts_2.TYPES.string,
                    value: startIndex > 0 ? searchString.substring(0, startIndex) : ''
                };
            },
            'substring-after': (search, searchFor) => {
                const searchString = Util_1.opFunc.getValueByType(this.getContext('substring-after') + '(first argument)', ['string'], search);
                const searchForString = Util_1.opFunc.getValueByType(this.getContext('substring-after') + '(second argument)', ['string'], searchFor);
                const startIndex = searchString.indexOf(searchForString);
                return {
                    type: consts_2.TYPES.string,
                    value: startIndex > 0 ? searchString.substring(startIndex + searchForString.length) : ''
                };
            },
            sum: (item) => {
                Util_1.opFunc.validate.areNumbers(item, this.getContext('sum'));
                return {
                    type: consts_2.TYPES.number,
                    value: item.value.reduce((r, node) => {
                        r += node[consts_1.KEYS.value];
                        return r;
                    }, 0)
                };
            },
            true: () => {
                return {
                    type: consts_2.TYPES.boolean,
                    value: true
                };
            }
        };
        this.pr = {
            setContextProperties: (props) => {
                this.contextProperties.isFilterMode = props === null || props === void 0 ? void 0 : props.isFilterMode;
                this.contextProperties.lastIndex = props === null || props === void 0 ? void 0 : props.lastIndex;
                this.contextProperties.firstIndex = props === null || props === void 0 ? void 0 : props.firstIndex;
                this.contextProperties.current = props === null || props === void 0 ? void 0 : props.current;
                this.contextProperties.index = props === null || props === void 0 ? void 0 : props.index;
                this.contextProperties.isLast = (props === null || props === void 0 ? void 0 : props.index) === (props === null || props === void 0 ? void 0 : props.lastIndex) || false;
                this.contextProperties.isFirst = (props === null || props === void 0 ? void 0 : props.index) === (props === null || props === void 0 ? void 0 : props.firstIndex) || false;
            }
        };
        this.mergeCustomFunction(customFunctions);
        // prerun, called each time before 
        this.prSequence = [
            this.pr.setContextProperties
        ];
    }
    ;
    preRun(props) {
        for (const [i, f] of this.prSequence.entries()) {
            f(props);
        }
    }
    ;
    run(fName, fArgs) {
        if (!this.mergedF[fName]) {
            (0, Util_1.throwError)(this.getContext(fName), `function "${fName}" does not exists`);
        }
        return this.mergedF[fName].apply(this, fArgs);
    }
    ;
    getFunctionNames() {
        return Object.keys(this.mergedF);
    }
    ;
}
exports.Functions = Functions;
;
//# sourceMappingURL=Functions.js.map