"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Operations = void 0;
const consts_1 = require("../consts");
const consts_2 = require("../nodes/consts");
const Util_1 = require("../Util");
class Operations {
    getContext(operator) {
        return `Operations[${operator}]`;
    }
    ;
    constructor() {
        this.props = {
            leftOperandItem: null,
            rightOperandItem: null,
            context: null,
            set: (props) => {
                this.props.leftOperandItem = props.leftOperandItem;
                this.props.rightOperandItem = props.rightOperandItem;
                this.props.context = this.getContext(props.context);
            },
            reset: () => {
                this.props.leftOperandItem = null;
                this.props.rightOperandItem = null;
                this.props.context = null;
            }
        };
        this.tests = {
            isNodes: (operand) => {
                return operand.type === consts_1.TYPES.nodes;
            },
            isNodeObjectType: (operand) => {
                return [consts_2.SYMBOLS.array, consts_2.SYMBOLS.object].includes(operand.value);
            },
            isOperatedValues: (operand) => {
                return operand.type === 'operatedValues';
            },
            startedFromRootNode: (operand) => {
                return this.tests.isNodes(operand) && !!operand.startedFromRoot;
            },
            isPositionType: (operand) => {
                return operand.type === 'position';
            },
            operatedValues: {
                allOperandsAreNodes: (operatedValues) => {
                    return operatedValues.every(opv => opv.operands.every((operand) => {
                        return operand.type === consts_1.TYPES.nodes;
                    }));
                }
            },
            position: (operationType) => {
                let leftOperandItem = this.props.leftOperandItem;
                let rightOperandItem = this.props.rightOperandItem;
                if (this.tests.isPositionType(this.props.rightOperandItem)) {
                    const tmp = this.props.rightOperandItem;
                    rightOperandItem = this.props.leftOperandItem;
                    leftOperandItem = tmp;
                    switch (operationType) {
                        case '<':
                            operationType = '>=';
                            break;
                        case '>':
                            operationType = '<=';
                            break;
                        case '<=':
                            operationType = '>';
                            break;
                        case '>=':
                            operationType = '<';
                            break;
                    }
                }
                if (rightOperandItem.type !== 'number') {
                    (0, Util_1.throwError)(`Operations.processPosition[${operationType}]`, `Expected a number to test position against, instead got ${rightOperandItem.type}.`);
                }
                const operationTest = {
                    '=': () => leftOperandItem.value === rightOperandItem.value,
                    '!=': () => leftOperandItem.value !== rightOperandItem.value,
                    '>': () => leftOperandItem.value > rightOperandItem.value,
                    '>=': () => leftOperandItem.value >= rightOperandItem.value,
                    '<': () => leftOperandItem.value < rightOperandItem.value,
                    '<=': () => leftOperandItem.value <= rightOperandItem.value
                };
                return operationTest[operationType]() || false;
            },
            isInvalid: (operand) => {
                return operand.type === consts_1.TYPES.invalid;
            },
            areEqualNodes: () => {
                return this.tests.isNodes(this.props.leftOperandItem) &&
                    this.tests.isNodes(this.props.rightOperandItem) &&
                    this.props.leftOperandItem.value.length === this.props.rightOperandItem.value.length &&
                    this.props.leftOperandItem.value.length === 1 && // only accept 1 node
                    Util_1.nodesOps.tests.isEqual(this.props.leftOperandItem.value[0], this.props.rightOperandItem.value[0]);
            }
        };
        this.validate = {
            operandsExists: () => {
                if (!this.props.leftOperandItem) {
                    (0, Util_1.throwError)(this.props.context, `Left operand is invalid`);
                }
                if (!this.props.rightOperandItem) {
                    (0, Util_1.throwError)(this.props.context, `Right operand is invalid`);
                }
            },
            types: (types, shouldThrow = false) => {
                let message;
                if (!types.includes(this.props.leftOperandItem.type)) {
                    message = `Left operand is an invalid type. Was expecting ${types.join(' or ')} type but received ${this.props.leftOperandItem.type}`;
                    shouldThrow && (0, Util_1.throwError)(this.props.context, message);
                }
                if (!types.includes(this.props.rightOperandItem.type)) {
                    message = `Right operand is an invalid type. Was expecting ${types.join(' or ')} type but received ${this.props.rightOperandItem.type}`;
                    shouldThrow && (0, Util_1.throwError)(this.props.context, message);
                }
                if (message) {
                    return {
                        type: consts_1.TYPES.invalid,
                        value: message
                    };
                }
            },
            nodesLength: (expectedLength, symbol) => {
                if (this.tests.isNodes(this.props.leftOperandItem)) {
                    Util_1.opFunc.validate.numberOfNodeItems(this.props.leftOperandItem, expectedLength, symbol, this.props.context);
                }
                if (this.tests.isNodes(this.props.rightOperandItem)) {
                    Util_1.opFunc.validate.numberOfNodeItems(this.props.rightOperandItem, expectedLength, symbol, this.props.context);
                }
            }
        };
        this.keep = {
            _numberNodes: (operand) => {
                if (this.tests.isNodes(operand)) {
                    return {
                        type: operand.type,
                        value: operand.value.filter(li => li[consts_2.KEYS.valueType] === 'number')
                    };
                }
                return operand;
            },
            numberNodes: () => {
                this.props.leftOperandItem = this.keep._numberNodes(this.props.leftOperandItem);
                this.props.rightOperandItem = this.keep._numberNodes(this.props.rightOperandItem);
            },
            _numberOperatedValues: (operand) => {
                if (this.tests.isOperatedValues(operand)) {
                    return {
                        type: operand.type,
                        value: operand.value.filter(li => !isNaN(li.value))
                    };
                }
                return operand;
            },
            numberOperatedValues: () => {
                this.props.leftOperandItem = this.keep._numberOperatedValues(this.props.leftOperandItem);
                this.props.rightOperandItem = this.keep._numberOperatedValues(this.props.rightOperandItem);
            },
            notFromRootNodes: (operandList) => {
                return operandList.operands.filter(op => {
                    return !this.tests.startedFromRootNode(op);
                });
            }
        };
        this.convert = {
            _toOperatedValues: (operand) => {
                if (this.tests.isNodes(operand)) {
                    const leftStartedFromRoot = this.tests.startedFromRootNode(operand);
                    return {
                        type: 'operatedValues',
                        value: operand.value.map(li => {
                            return {
                                operands: [{ startedFromRoot: leftStartedFromRoot, type: consts_1.TYPES.nodes, value: li }],
                                value: li[consts_2.KEYS.value],
                                type: li[consts_2.KEYS.valueType]
                            };
                        })
                    };
                }
                else if (!this.tests.isOperatedValues(operand)) {
                    return {
                        type: 'operatedValues',
                        value: [
                            {
                                operands: [{ startedFromRoot: false, type: operand.type, value: operand.value }],
                                value: operand.value,
                                type: operand.type
                            }
                        ]
                    };
                }
                return operand;
            },
            toOperatedValues: () => {
                this.props.leftOperandItem = this.convert._toOperatedValues(this.props.leftOperandItem);
                this.props.rightOperandItem = this.convert._toOperatedValues(this.props.rightOperandItem);
            }
        };
        this.f = {
            filterMode: {
                '+': () => {
                    const invalidType = this.validate.types(['number', consts_1.TYPES.nodes, 'operatedValues']);
                    if (invalidType) {
                        return invalidType;
                    }
                    ;
                    this.keep.numberNodes();
                    this.keep.numberOperatedValues();
                    this.convert.toOperatedValues();
                    const type = 'operatedValues';
                    let operand1 = this.props.leftOperandItem;
                    let operand2 = this.props.rightOperandItem;
                    if (!operand1.value.length && operand2.value.length) {
                        const tmp = operand1;
                        operand1 = operand2;
                        operand2 = tmp;
                    }
                    const value = operand1.value.reduce((r, lov) => {
                        if (!operand2.value.length) {
                            r.push({
                                operands: [...lov.operands],
                                value: lov.value,
                                type: consts_1.TYPES.number
                            });
                        }
                        operand2.value.forEach(rov => {
                            r.push({
                                operands: [...lov.operands, ...rov.operands],
                                value: lov.value + rov.value,
                                type: consts_1.TYPES.number
                            });
                        });
                        return r;
                    }, []);
                    return {
                        type: type,
                        value: value
                    };
                },
                '-': () => {
                    const invalidType = this.validate.types(['number', consts_1.TYPES.nodes, 'operatedValues']);
                    if (invalidType) {
                        return invalidType;
                    }
                    ;
                    this.keep.numberNodes();
                    this.keep.numberOperatedValues();
                    this.convert.toOperatedValues();
                    const type = 'operatedValues';
                    let value;
                    if (!this.props.leftOperandItem.value.length && this.props.rightOperandItem.value.length) {
                        value = this.props.rightOperandItem.value.reduce((r, rov) => {
                            r.push({
                                operands: [...rov.operands],
                                value: 0 - rov.value,
                                type: consts_1.TYPES.number
                            });
                            return r;
                        }, []);
                    }
                    else if (this.props.leftOperandItem.value.length) {
                        value = this.props.leftOperandItem.value.reduce((r, lov) => {
                            if (!this.props.rightOperandItem.value.length) {
                                r.push({
                                    operands: [...lov.operands],
                                    value: lov.value,
                                    type: consts_1.TYPES.number
                                });
                            }
                            this.props.rightOperandItem.value.forEach(rov => {
                                r.push({
                                    operands: [...lov.operands, ...rov.operands],
                                    value: lov.value - rov.value,
                                    type: consts_1.TYPES.number
                                });
                            });
                            return r;
                        }, []);
                    }
                    return {
                        type: type,
                        value: value
                    };
                },
                '*': () => {
                    const invalidType = this.validate.types(['number', consts_1.TYPES.nodes, 'operatedValues']);
                    if (invalidType) {
                        return invalidType;
                    }
                    ;
                    this.keep.numberNodes();
                    this.keep.numberOperatedValues();
                    this.convert.toOperatedValues();
                    const type = 'operatedValues';
                    let operand1 = this.props.leftOperandItem;
                    let operand2 = this.props.rightOperandItem;
                    if (!operand1.value.length && operand2.value.length) {
                        const tmp = operand1;
                        operand1 = operand2;
                        operand2 = tmp;
                    }
                    const value = operand1.value.reduce((r, lov) => {
                        if (!operand2.value.length) {
                            r.push({
                                operands: [...lov.operands],
                                value: 0,
                                type: consts_1.TYPES.number
                            });
                        }
                        operand2.value.forEach(rov => {
                            r.push({
                                operands: [...lov.operands, ...rov.operands],
                                value: lov.value * rov.value,
                                type: consts_1.TYPES.number
                            });
                        });
                        return r;
                    }, []);
                    return {
                        type: type,
                        value: value
                    };
                },
                'div': () => {
                    const invalidType = this.validate.types(['number', consts_1.TYPES.nodes, 'operatedValues']);
                    if (invalidType) {
                        return invalidType;
                    }
                    ;
                    this.keep.numberNodes();
                    this.keep.numberOperatedValues();
                    this.convert.toOperatedValues();
                    let type = consts_1.TYPES.operatedValues;
                    let value;
                    if (!this.props.leftOperandItem.value.length) {
                        type = consts_1.TYPES.invalid;
                        value = 'Left operand is an invalid. It has no valid value';
                    }
                    else if (this.props.leftOperandItem.value.length) {
                        value = this.props.leftOperandItem.value.reduce((r, lov) => {
                            if (!this.props.rightOperandItem.value.length) {
                                r.push({
                                    operands: [...lov.operands],
                                    value: 0,
                                    type: consts_1.TYPES.number
                                });
                            }
                            this.props.rightOperandItem.value.forEach(rov => {
                                r.push({
                                    operands: [...lov.operands, ...rov.operands],
                                    value: lov.value / rov.value,
                                    type: consts_1.TYPES.number
                                });
                            });
                            return r;
                        }, []);
                    }
                    return {
                        type: type,
                        value: value
                    };
                },
                'mod': () => {
                    const invalidType = this.validate.types(['number', consts_1.TYPES.nodes, 'operatedValues']);
                    if (invalidType) {
                        return invalidType;
                    }
                    ;
                    this.keep.numberNodes();
                    this.keep.numberOperatedValues();
                    this.convert.toOperatedValues();
                    const type = 'operatedValues';
                    let value;
                    if (!this.props.leftOperandItem.value.length && this.props.rightOperandItem.value.length) {
                        value = this.props.rightOperandItem.value.reduce((r, rov) => {
                            r.push({
                                operands: [...rov.operands],
                                value: 0 % rov.value,
                                type: consts_1.TYPES.number
                            });
                            return r;
                        }, []);
                    }
                    else if (this.props.leftOperandItem.value.length) {
                        value = this.props.leftOperandItem.value.reduce((r, lov) => {
                            if (!this.props.rightOperandItem.value.length) {
                                r.push({
                                    operands: [...lov.operands],
                                    value: lov.value,
                                    type: consts_1.TYPES.number
                                });
                            }
                            this.props.rightOperandItem.value.forEach(rov => {
                                r.push({
                                    operands: [...lov.operands, ...rov.operands],
                                    value: lov.value % rov.value,
                                    type: consts_1.TYPES.number
                                });
                            });
                            return r;
                        }, []);
                    }
                    return {
                        type: type,
                        value: value
                    };
                },
                '=': () => {
                    if (this.tests.isInvalid(this.props.leftOperandItem)) {
                        return this.props.leftOperandItem;
                    }
                    ;
                    if (this.tests.isInvalid(this.props.rightOperandItem)) {
                        return this.props.rightOperandItem;
                    }
                    // TODO type = variable and
                    let type = consts_1.TYPES.boolean;
                    let value = false;
                    if (this.tests.isPositionType(this.props.leftOperandItem) || this.tests.isPositionType(this.props.rightOperandItem)) {
                        value = this.tests.position('=');
                    }
                    else if (this.tests.areEqualNodes()) {
                        value = true;
                    }
                    else {
                        this.convert.toOperatedValues();
                        value = this.props.leftOperandItem.value.some(lo => {
                            return this.props.rightOperandItem.value.some(ro => {
                                return lo.type === ro.type && lo.value === ro.value;
                            });
                        });
                    }
                    return {
                        type: type,
                        value: value
                    };
                },
                '>': () => {
                    if (this.tests.isInvalid(this.props.leftOperandItem)) {
                        return this.props.leftOperandItem;
                    }
                    ;
                    if (this.tests.isInvalid(this.props.rightOperandItem)) {
                        return this.props.rightOperandItem;
                    }
                    let type = consts_1.TYPES.boolean;
                    let value = false;
                    this.keep.numberNodes();
                    if (this.tests.isPositionType(this.props.leftOperandItem) || this.tests.isPositionType(this.props.rightOperandItem)) {
                        value = this.tests.position('>');
                    }
                    else {
                        this.convert.toOperatedValues();
                        this.keep.numberOperatedValues();
                        value = this.props.leftOperandItem.value.some(lo => {
                            return this.props.rightOperandItem.value.some(ro => {
                                return lo.type === ro.type && lo.value > ro.value;
                            });
                        });
                    }
                    return {
                        type: type,
                        value: value
                    };
                },
                '<': () => {
                    if (this.tests.isInvalid(this.props.leftOperandItem)) {
                        return this.props.leftOperandItem;
                    }
                    ;
                    if (this.tests.isInvalid(this.props.rightOperandItem)) {
                        return this.props.rightOperandItem;
                    }
                    let type = consts_1.TYPES.boolean;
                    let value = false;
                    this.keep.numberNodes();
                    if (this.tests.isPositionType(this.props.leftOperandItem) || this.tests.isPositionType(this.props.rightOperandItem)) {
                        value = this.tests.position('>');
                    }
                    else {
                        this.convert.toOperatedValues();
                        this.keep.numberOperatedValues();
                        value = this.props.leftOperandItem.value.some(lo => {
                            return this.props.rightOperandItem.value.some(ro => {
                                return lo.type === ro.type && lo.value < ro.value;
                            });
                        });
                    }
                    return {
                        type: type,
                        value: value
                    };
                },
                '>=': () => {
                    if (this.tests.isInvalid(this.props.leftOperandItem)) {
                        return this.props.leftOperandItem;
                    }
                    ;
                    if (this.tests.isInvalid(this.props.rightOperandItem)) {
                        return this.props.rightOperandItem;
                    }
                    let type = consts_1.TYPES.boolean;
                    let value = false;
                    this.keep.numberNodes();
                    if (this.tests.isPositionType(this.props.leftOperandItem) || this.tests.isPositionType(this.props.rightOperandItem)) {
                        value = this.tests.position('>');
                    }
                    else {
                        this.convert.toOperatedValues();
                        this.keep.numberOperatedValues();
                        value = this.props.leftOperandItem.value.some(lo => {
                            return this.props.rightOperandItem.value.some(ro => {
                                return lo.type === ro.type && lo.value >= ro.value;
                            });
                        });
                    }
                    return {
                        type: type,
                        value: value
                    };
                },
                '<=': () => {
                    if (this.tests.isInvalid(this.props.leftOperandItem)) {
                        return this.props.leftOperandItem;
                    }
                    ;
                    if (this.tests.isInvalid(this.props.rightOperandItem)) {
                        return this.props.rightOperandItem;
                    }
                    let type = consts_1.TYPES.boolean;
                    let value = false;
                    this.keep.numberNodes();
                    if (this.tests.isPositionType(this.props.leftOperandItem) || this.tests.isPositionType(this.props.rightOperandItem)) {
                        value = this.tests.position('>');
                    }
                    else {
                        this.convert.toOperatedValues();
                        this.keep.numberOperatedValues();
                        value = this.props.leftOperandItem.value.some(lo => {
                            return this.props.rightOperandItem.value.some(ro => {
                                return lo.type === ro.type && lo.value <= ro.value;
                            });
                        });
                    }
                    return {
                        type: type,
                        value: value
                    };
                },
                'and': () => {
                    if (this.tests.isInvalid(this.props.leftOperandItem)) {
                        return this.props.leftOperandItem;
                    }
                    ;
                    if (this.tests.isInvalid(this.props.rightOperandItem)) {
                        return this.props.rightOperandItem;
                    }
                    const lbool = Array.isArray(this.props.leftOperandItem.value) ? !!this.props.leftOperandItem.value.length : !!this.props.leftOperandItem.value;
                    const rbool = Array.isArray(this.props.rightOperandItem.value) ? !!this.props.rightOperandItem.value.length : !!this.props.rightOperandItem.value;
                    return {
                        type: consts_1.TYPES.boolean,
                        value: lbool && rbool
                    };
                },
                'or': () => {
                    if (this.tests.isInvalid(this.props.leftOperandItem)) {
                        return this.props.leftOperandItem;
                    }
                    ;
                    if (this.tests.isInvalid(this.props.rightOperandItem)) {
                        return this.props.rightOperandItem;
                    }
                    const lbool = Array.isArray(this.props.leftOperandItem.value) ? !!this.props.leftOperandItem.value.length : !!this.props.leftOperandItem.value;
                    const rbool = Array.isArray(this.props.rightOperandItem.value) ? !!this.props.rightOperandItem.value.length : !!this.props.rightOperandItem.value;
                    return {
                        type: consts_1.TYPES.boolean,
                        value: lbool || rbool
                    };
                },
                '|': () => {
                    if (this.tests.isInvalid(this.props.leftOperandItem)) {
                        return this.props.leftOperandItem;
                    }
                    ;
                    if (this.tests.isInvalid(this.props.rightOperandItem)) {
                        return this.props.rightOperandItem;
                    }
                    const invalidType = this.validate.types([consts_1.TYPES.nodes]);
                    if (invalidType) {
                        return invalidType;
                    }
                    return {
                        type: consts_1.TYPES.nodes,
                        value: [...this.props.leftOperandItem.value, ...this.props.rightOperandItem.value.filter(rv => {
                                return !this.props.leftOperandItem.value.some(lv => lv[consts_2.KEYS.id] === rv[consts_2.KEYS.id]);
                            })]
                    };
                }
            },
            default: {
                '+': () => {
                    const leftValue = Util_1.opFunc.getValueByType(this.getContext('+'), [consts_1.TYPES.number], this.props.leftOperandItem);
                    const rightValue = Util_1.opFunc.getValueByType(this.getContext('+'), [consts_1.TYPES.number], this.props.rightOperandItem);
                    return {
                        type: consts_1.TYPES.number,
                        value: leftValue + rightValue
                    };
                },
                '-': () => {
                    const leftValue = Util_1.opFunc.getValueByType(this.getContext('-'), [consts_1.TYPES.number], this.props.leftOperandItem);
                    const rightValue = Util_1.opFunc.getValueByType(this.getContext('-'), [consts_1.TYPES.number], this.props.rightOperandItem);
                    return {
                        type: consts_1.TYPES.number,
                        value: leftValue - rightValue
                    };
                },
                '*': () => {
                    const leftValue = Util_1.opFunc.getValueByType(this.getContext('*'), [consts_1.TYPES.number], this.props.leftOperandItem);
                    const rightValue = Util_1.opFunc.getValueByType(this.getContext('*'), [consts_1.TYPES.number], this.props.rightOperandItem);
                    return {
                        type: consts_1.TYPES.number,
                        value: leftValue * rightValue
                    };
                },
                'div': () => {
                    const leftValue = Util_1.opFunc.getValueByType(this.getContext('div'), [consts_1.TYPES.number], this.props.leftOperandItem);
                    const rightValue = Util_1.opFunc.getValueByType(this.getContext('div'), [consts_1.TYPES.number], this.props.rightOperandItem);
                    return {
                        type: consts_1.TYPES.number,
                        value: leftValue / rightValue
                    };
                },
                'mod': () => {
                    const leftValue = Util_1.opFunc.getValueByType(this.getContext('mod'), [consts_1.TYPES.number], this.props.leftOperandItem);
                    const rightValue = Util_1.opFunc.getValueByType(this.getContext('mod'), [consts_1.TYPES.number], this.props.rightOperandItem);
                    return {
                        type: consts_1.TYPES.number,
                        value: leftValue % rightValue
                    };
                },
                '=': () => {
                    return this.f.filterMode['=']();
                },
                '>': () => {
                    return this.f.filterMode['>']();
                },
                '<': () => {
                    return this.f.filterMode['<']();
                },
                '>=': () => {
                    return this.f.filterMode['>=']();
                },
                '<=': () => {
                    return this.f.filterMode['<=']();
                },
                'and': () => {
                    return this.f.filterMode['and']();
                },
                'or': () => {
                    return this.f.filterMode['or']();
                },
                '|': () => {
                    return this.f.filterMode['|']();
                }
            }
        };
    }
    run(opName, leftOperandItem, rightOperandItem, { isFilterMode }) {
        this.props.set({ leftOperandItem, rightOperandItem, context: opName });
        const mode = isFilterMode ? 'filterMode' : 'default';
        if (!this.f[mode]) {
            (0, Util_1.throwError)(this.getContext('run'), `No operation "${opName}" exists.`);
        }
        return this.f[mode][opName]();
    }
}
exports.Operations = Operations;
//# sourceMappingURL=Operations.js.map