"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Variables = void 0;
const consts_1 = require("../consts");
const Nodes_1 = require("../nodes/Nodes");
const Util_1 = require("../Util");
class Variables {
    constructor(rootProps, variables) {
        this.variables = variables;
        this.tests = {
            isPath: (value) => {
                return typeof value === 'string' && value.includes('$');
            }
        };
        this.NodesClass = new Nodes_1.Nodes();
        this.variables = variables || {};
        this.cacheVariables(rootProps);
    }
    cacheVariables({ root, nodes }) {
        this.cache = {
            root: {
                name: 'root',
                value: root,
                nodes,
                isPath: false
            }
        };
        for (let key in this.variables) {
            this.cache[key] = {
                name: key,
                value: this.variables[key],
                nodes: null,
                isPath: this.tests.isPath(this.variables[key])
            };
        }
    }
    ;
    getVariableEndIndex(path, startIndex) {
        const stopChars = [' ', '[', ']', '/', ...consts_1.OPERATORS.byLength['1']];
        for (let i = startIndex; i < path.length; i++) {
            if (stopChars.includes(path[i])) {
                return i;
            }
        }
        return startIndex;
    }
    ;
    getReferenceVariableInPath(variableValue) {
        const startOfDollarSign = variableValue.indexOf('$');
        const endOfVariableName = this.getVariableEndIndex(variableValue, startOfDollarSign);
        return variableValue.substring(startOfDollarSign + 1, endOfVariableName);
    }
    ;
    getVariablePathResult(path) {
        const postfixPath = this.postfixInstance.toPostfix(path);
        this.processorInstance.processPostfixPath(postfixPath);
        const item = this.processorInstance.output.pop();
        let value;
        if (item.type === consts_1.TYPES.nodes) {
            value = this.toJsonInstance.reconstruct(item.value);
            if (value.length === 1) {
                value = value[0];
            }
        }
        else {
            value = item.value;
        }
        return value;
    }
    ;
    updateVariablePathValue(variableCache) {
        const variableReferenceName = this.getReferenceVariableInPath(variableCache.value);
        if (!this.cache[variableReferenceName]) {
            throw new Error(`[Variables.updateVariablePathValue] variable \$${variableReferenceName} is not defined`);
        }
        if (this.cache[variableReferenceName].isPath) {
            this.updateVariablePathValue(this.cache[variableReferenceName]);
        }
        if (!this.cache[variableReferenceName].nodes) {
            const referenceValue = this.cache[variableReferenceName].value;
            this.cache[variableReferenceName].nodes = this.NodesClass.jsonToNodes(referenceValue, this.cache[variableReferenceName].name);
        }
        this.cache[variableCache.name].value = this.getVariablePathResult(variableCache.value);
        this.cache[variableCache.name].isPath = false;
    }
    ;
    // This should be called early
    setInstances({ postfixInstance, processorInstance, toJsonInstance }) {
        this.postfixInstance = postfixInstance;
        this.processorInstance = processorInstance;
        this.toJsonInstance = toJsonInstance;
    }
    ;
    getVariableRootNode(variable) {
        if (variable[0] === '$') {
            variable = variable.substring(1);
        }
        if (!this.cache[variable]) {
            throw new Error(`[Variables.getVariableRootNode] variable \$${variable} is not defined`);
        }
        if (this.cache[variable].isPath) {
            this.updateVariablePathValue(this.cache[variable]);
        }
        if (!this.cache[variable].nodes) {
            this.cache[variable].nodes = this.NodesClass.jsonToNodes(this.cache[variable].value, variable);
        }
        return Util_1.nodesOps.get.root(this.cache[variable].nodes);
    }
    ;
    reset() {
        this.variables = null;
        this.cache = null;
    }
    ;
}
exports.Variables = Variables;
//# sourceMappingURL=Variables.js.map