"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runPath = exports.runPaths = exports.eStackTypesObject = exports.KEYS = exports.nodesOps = void 0;
const RunPath_1 = require("./RunPath");
var Util_1 = require("./Util");
Object.defineProperty(exports, "nodesOps", { enumerable: true, get: function () { return Util_1.nodesOps; } });
var consts_1 = require("./nodes/consts");
Object.defineProperty(exports, "KEYS", { enumerable: true, get: function () { return consts_1.KEYS; } });
var types_1 = require("./types");
Object.defineProperty(exports, "eStackTypesObject", { enumerable: true, get: function () { return types_1.eStackTypesObject; } });
/**
 * Given a list of objects containing path and callback function,
 * for each object, run the defined path against the passed in json
 * and pass the result { path, value, error } into the callback function
 *
 * @param paths tPathWithCallBack[]: list of objects at minimum contains
 *    the path and callback "then" function
 * @param input tRunPathsInput: object containing the json,
 *    and the optional custom functions and variables declarations
 */
const runPaths = (paths, inputs) => {
    const runPathInstance = new RunPath_1.RunPath(inputs);
    paths.map((pathProp) => {
        if (pathProp.active === false) {
            return;
        }
        runPathInstance.callbackMode(pathProp);
    });
    runPathInstance.reset();
};
exports.runPaths = runPaths;
/**
 * If path is a string, run it against the passed in json and return the result
 *
 * If path is an object containing path and callback function, run the defined
 * path against the passed in json and pass the result { path, value, error }
 * into the callback function.
 *
 * @param path String|tPathWithCallBack: a string path or an object at minimum contains
 *    the path and callback "then" function
 * @param input tRunPathsInput: object containing the json, and the optional
 *    custom functions and variables declarations
 * @returns JSON[] if path is a string
 */
const runPath = (path, inputs) => {
    const runPathInstance = new RunPath_1.RunPath(inputs);
    const useCallBackOption = typeof path === 'string' ? false : true;
    if (useCallBackOption) {
        runPathInstance.callbackMode(path);
        runPathInstance.reset();
    }
    else {
        const result = runPathInstance.stringPathMode(path);
        runPathInstance.reset();
        return result;
    }
};
exports.runPath = runPath;
//# sourceMappingURL=index.js.map