"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Json = void 0;
const consts_1 = require("./consts");
const Util_1 = require("../Util");
class Json {
    constructor() {
        this.reconstruct = (nodes) => {
            return nodes.map((node) => {
                if (!Util_1.nodesOps.tests.exists(node[consts_1.KEYS.id])) {
                    return;
                }
                else if (node[consts_1.KEYS.value] === consts_1.SYMBOLS.array) {
                    return this.build.array(node);
                }
                else if (node[consts_1.KEYS.value] === consts_1.SYMBOLS.object) {
                    return this.build.object(node);
                }
                else {
                    return node[consts_1.KEYS.value];
                }
            });
        };
        this.build = {
            array: (node) => {
                const childrenNodes = Util_1.nodesOps.get.children(node);
                if (!childrenNodes.length) {
                    return [];
                }
                const result = [];
                childrenNodes.forEach((c) => {
                    if (c[consts_1.KEYS.group] !== consts_1.SYMBOLS.na && result[c[consts_1.KEYS.arrayPosition]] === undefined) {
                        result[c[consts_1.KEYS.arrayPosition]] = {};
                    }
                    let value = c[consts_1.KEYS.value];
                    if (value === consts_1.SYMBOLS.array) {
                        value = this.build.array(c);
                    }
                    else if (value === consts_1.SYMBOLS.object) {
                        value = this.build.object(c);
                    }
                    if (this.isValidKey(c[consts_1.KEYS.name])) {
                        result[c[consts_1.KEYS.arrayPosition]][c[consts_1.KEYS.name]] = value;
                    }
                    else {
                        result[c[consts_1.KEYS.arrayPosition]] = value;
                    }
                });
                return result;
            },
            object: (node) => {
                const childrenNodes = Util_1.nodesOps.get.children(node);
                if (!childrenNodes.length) {
                    return {};
                }
                const result = {};
                childrenNodes.forEach(node => {
                    const value = node[consts_1.KEYS.value];
                    if (value === consts_1.SYMBOLS.array) {
                        result[node[consts_1.KEYS.name]] = this.build.array(node);
                    }
                    else if (value === consts_1.SYMBOLS.object) {
                        result[node[consts_1.KEYS.name]] = this.build.object(node);
                    }
                    else {
                        result[node[consts_1.KEYS.name]] = value;
                    }
                });
                return result;
            },
        };
        this.isValidKey = (key) => {
            return !Object.values(consts_1.SYMBOLS).includes(key);
        };
    }
}
exports.Json = Json;
;
//# sourceMappingURL=Json.js.map