"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Nodes = void 0;
const State_1 = require("./State");
const consts_1 = require("./consts");
const consts_2 = require("../consts");
class Nodes {
    constructor() {
        this.add = {
            root: (data) => {
                return this.add.node(this.nodeState.getId(), consts_1.SYMBOLS.na, consts_1.SYMBOLS.root, data, null);
            },
            node: (id, group, key, value, parent, position) => {
                const depth = (parent ? parent[consts_1.KEYS.depth] : -1) + 1;
                const node = [id, depth, group, (position || position === 0 ? position : consts_1.SYMBOLS.na), key, value, this.determineValueType(value), {}];
                this.add.links(parent, node);
                this.nodeState.addNode(node);
                this.nodeState.incrementId();
                return node;
            },
            // Add parent/ancestor/children/descendant links
            links: (ancestorNode, currentNode) => {
                if (currentNode[consts_1.KEYS.links].parentId === undefined) {
                    currentNode[consts_1.KEYS.links] = {
                        ancestorIds: [],
                        parentId: ancestorNode ? ancestorNode[consts_1.KEYS.id] : null,
                        descendantIds: [],
                        siblings: [],
                        childrenIds: []
                    };
                }
                if (ancestorNode) {
                    if (currentNode[consts_1.KEYS.links].parentId === ancestorNode[consts_1.KEYS.id]) {
                        ancestorNode[consts_1.KEYS.links].childrenIds.push(currentNode[consts_1.KEYS.id]);
                    }
                    ancestorNode[consts_1.KEYS.links].descendantIds.push(currentNode[consts_1.KEYS.id]);
                    currentNode[consts_1.KEYS.links].ancestorIds.push(ancestorNode[consts_1.KEYS.id]);
                    this.add.links(this.nodeState.getNodeById(ancestorNode[consts_1.KEYS.links].parentId), currentNode);
                }
            },
            siblings: (nodes) => {
                nodes.forEach(node => {
                    nodes.forEach(linkedNode => {
                        if (node[consts_1.KEYS.id] !== linkedNode[consts_1.KEYS.id]) {
                            node[consts_1.KEYS.links].siblings.push(linkedNode[consts_1.KEYS.id]);
                        }
                    });
                });
            }
        };
        this.deconstruct = {
            array: (data, parent) => {
                data.forEach((d, i) => {
                    const value = this.deconstruct.getValue(d);
                    if (value === consts_1.SYMBOLS.array) {
                        const id = this.nodeState.getId();
                        const node = this.add.node(id, consts_1.SYMBOLS.na, consts_1.SYMBOLS.na, consts_1.SYMBOLS.array, parent, i);
                        this.deconstruct.array(d, node);
                    }
                    else if (value === consts_1.SYMBOLS.object) {
                        // create a meta object
                        const currentId = this.nodeState.getId();
                        const node = this.add.node(currentId, consts_1.SYMBOLS.na, consts_1.SYMBOLS.ao, consts_1.SYMBOLS.object, parent, i);
                        this.deconstruct.object(d, node, i);
                    }
                    else {
                        this.add.node(this.nodeState.getId(), consts_1.SYMBOLS.na, consts_1.SYMBOLS.value, d, parent, i);
                    }
                });
            },
            object: (data, parent, pos) => {
                const group = this.nodeState.incrementGroup();
                const siblings = [];
                for (let key in data) {
                    const currentId = this.nodeState.getId();
                    const value = this.deconstruct.getValue(data[key]);
                    const node = this.add.node(currentId, group, key, value, parent, pos);
                    if (value === consts_1.SYMBOLS.array) {
                        this.deconstruct.array(data[key], node);
                    }
                    else if (value === consts_1.SYMBOLS.object) {
                        this.deconstruct.object(data[key], node);
                    }
                    else {
                        // this.add(b.id, currentId, group, symbols.value, data[key], pos);
                    }
                    siblings.push(node);
                }
                this.add.siblings(siblings);
            },
            getValue: (value) => {
                if (Array.isArray(value)) {
                    return consts_1.SYMBOLS.array;
                }
                else if (value && typeof value === 'object') {
                    return consts_1.SYMBOLS.object;
                }
                return value;
            }
        };
        this.determineValueType = (value) => {
            if (value === consts_1.SYMBOLS.array) {
                return 'array';
            }
            else if (value === consts_1.SYMBOLS.object) {
                return 'object';
            }
            else if (value === null) {
                return 'null';
            }
            else if (value === undefined) {
                return 'undefined';
            }
            else if (typeof value === consts_2.TYPES.string) {
                return consts_2.TYPES.string;
            }
            else if (typeof value === consts_2.TYPES.boolean) {
                return consts_2.TYPES.boolean;
            }
            else if (!Number.isNaN(Number(value))) {
                return 'number';
            }
        };
        // singleton nodeState shared across main and variables
        this.nodeState = State_1.NodesState.getInstance();
    }
    jsonToNodes(data, caller) {
        this.nodeState.setCaller(caller);
        const value = this.deconstruct.getValue(data);
        const root = this.add.root(value);
        if (value === consts_1.SYMBOLS.array) {
            this.deconstruct.array(data, root);
        }
        else if (value === consts_1.SYMBOLS.object) {
            this.deconstruct.object(data, root);
        }
        return this.nodeState.getNodes();
    }
    ;
}
exports.Nodes = Nodes;
//# sourceMappingURL=Nodes.js.map