"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const consts_1 = require("../consts");
const Functions_1 = require("../evaluate/Functions");
const Postfix_1 = require("./Postfix");
describe('Class Postfix', () => {
    let callerId, expected, functionsInstance, path, postfixInstance;
    beforeEach(() => {
        callerId = 'main';
        functionsInstance = new Functions_1.Functions();
    });
    describe('Simple path', () => {
        beforeEach(() => {
            postfixInstance = new Postfix_1.Postfix(callerId, functionsInstance);
        });
        it('path is /a', () => {
            path = '/a',
                expect(postfixInstance.toPostfix(path)).toEqual([
                    {
                        type: consts_1.TYPES.rootPath,
                        callerId,
                        value: '/a'
                    }
                ]);
        });
    });
    describe('Simple predicate', () => {
        beforeEach(() => {
            postfixInstance = new Postfix_1.Postfix(callerId, functionsInstance);
        });
        it('path is /a[b]', () => {
            path = '/a[b]',
                expect(postfixInstance.toPostfix(path)).toEqual([
                    {
                        type: consts_1.TYPES.rootPath,
                        callerId,
                        value: '/a'
                    },
                    [
                        {
                            type: consts_1.TYPES.path,
                            callerId,
                            value: './b'
                        }
                    ]
                ]);
        });
        it('path is /a[b]/c', () => {
            path = '/a[b]/c',
                expect(postfixInstance.toPostfix(path)).toEqual([
                    {
                        type: consts_1.TYPES.rootPath,
                        callerId,
                        value: '/a'
                    },
                    [
                        {
                            type: consts_1.TYPES.path,
                            callerId,
                            value: './b'
                        }
                    ],
                    {
                        type: consts_1.TYPES.path,
                        callerId,
                        value: '/c'
                    }
                ]);
        });
    });
    describe('Operators', () => {
        beforeEach(() => {
            postfixInstance = new Postfix_1.Postfix(callerId, functionsInstance);
        });
        it('path is /a/b = /a/c', () => {
            path = '/a/b = /a/c';
            expected = [
                {
                    type: consts_1.TYPES.rootPath,
                    callerId,
                    value: '/a/b'
                }, {
                    type: consts_1.TYPES.rootPath,
                    callerId,
                    value: '/a/c'
                },
                {
                    type: consts_1.TYPES.operator,
                    callerId,
                    value: '='
                }
            ];
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
            path = '/a/b=/a/c';
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
            path = '/a/b =/a/c';
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
            path = '/a/b=  /a/c';
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
        });
        it('path is /a[b + /c/d = 3]', () => {
            path = '/a[b + /c/d = 3]';
            expected = [
                {
                    callerId,
                    type: consts_1.TYPES.rootPath,
                    value: '/a'
                },
                [
                    {
                        callerId,
                        type: consts_1.TYPES.path,
                        value: './b'
                    },
                    {
                        callerId,
                        type: consts_1.TYPES.rootPath,
                        value: '/c/d'
                    },
                    {
                        callerId,
                        type: consts_1.TYPES.operator,
                        value: '+'
                    },
                    {
                        type: consts_1.TYPES.number,
                        value: 3
                    },
                    {
                        callerId,
                        type: consts_1.TYPES.operator,
                        value: '='
                    }
                ]
            ];
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
            path = '/a[b+/c/d=3]';
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
            path = '/a[ b+ /c/d= 3  ]';
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
        });
        it('path is 1 - 2 * 3', () => {
            path = '1 - 2 * 3';
            const expected = [
                {
                    type: consts_1.TYPES.number,
                    value: 1
                },
                {
                    type: consts_1.TYPES.number,
                    value: 2
                },
                {
                    type: consts_1.TYPES.number,
                    value: 3
                },
                {
                    callerId,
                    type: consts_1.TYPES.operator,
                    value: '*'
                },
                {
                    callerId,
                    type: consts_1.TYPES.operator,
                    value: '-'
                }
            ];
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
            path = '1-2*3';
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
            path = '1 -2 *3';
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
        });
        it('path is (1 - -2) * 3]', () => {
            path = '(1 - -2) * 3';
            const expected = [
                {
                    type: consts_1.TYPES.number,
                    value: 1
                },
                {
                    type: consts_1.TYPES.number,
                    value: -2
                },
                {
                    callerId,
                    type: consts_1.TYPES.operator,
                    value: '-'
                },
                {
                    type: consts_1.TYPES.number,
                    value: 3
                },
                {
                    callerId,
                    type: consts_1.TYPES.operator,
                    value: '*'
                }
            ];
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
            path = '(1--2)*   3';
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
        });
        it('path is /a[b + /c/d * 3]', () => {
            path = '/a[b + /c/d * 3]';
            expect(postfixInstance.toPostfix(path)).toEqual([
                {
                    callerId,
                    type: consts_1.TYPES.rootPath,
                    value: '/a'
                },
                [
                    {
                        callerId,
                        type: consts_1.TYPES.path,
                        value: './b'
                    },
                    {
                        callerId,
                        type: consts_1.TYPES.rootPath,
                        value: '/c/d'
                    },
                    {
                        type: consts_1.TYPES.number,
                        value: 3
                    },
                    {
                        callerId,
                        type: consts_1.TYPES.operator,
                        value: '*'
                    },
                    {
                        callerId,
                        type: consts_1.TYPES.operator,
                        value: '+'
                    }
                ]
            ]);
        });
    });
    describe('Functions', () => {
        beforeEach(() => {
            postfixInstance = new Postfix_1.Postfix(callerId, functionsInstance);
        });
        it('path is string(/a/b)', () => {
            path = 'string(/a/b)';
            expect(postfixInstance.toPostfix(path)).toEqual([
                {
                    type: consts_1.TYPES.arguments,
                    value: [{
                            type: consts_1.TYPES.rootPath,
                            callerId,
                            value: '/a/b'
                        }]
                },
                {
                    type: consts_1.TYPES.function,
                    callerId,
                    value: 'string'
                }
            ]);
        });
        it('path is string(/a[b=12]/c)', () => {
            path = 'string(/a[b=12]/c)';
            expect(postfixInstance.toPostfix(path)).toEqual([
                {
                    type: consts_1.TYPES.arguments,
                    value: [[
                            {
                                callerId,
                                type: consts_1.TYPES.rootPath,
                                value: '/a'
                            },
                            [
                                {
                                    callerId,
                                    type: consts_1.TYPES.path,
                                    value: './b'
                                },
                                {
                                    type: consts_1.TYPES.number,
                                    value: 12
                                },
                                {
                                    callerId,
                                    type: consts_1.TYPES.operator,
                                    value: '='
                                }
                            ],
                            {
                                callerId,
                                type: consts_1.TYPES.path,
                                value: '/c'
                            }
                        ]]
                },
                {
                    type: consts_1.TYPES.function,
                    callerId,
                    value: 'string'
                }
            ]);
        });
        it('path is /a[contains(., "test")]', () => {
            path = '/a[contains(., "test")]';
            expect(postfixInstance.toPostfix(path)).toEqual([
                {
                    type: consts_1.TYPES.rootPath,
                    callerId,
                    value: '/a'
                },
                [
                    {
                        type: consts_1.TYPES.arguments,
                        value: [
                            {
                                type: consts_1.TYPES.path,
                                callerId,
                                value: '.'
                            },
                            {
                                type: consts_1.TYPES.string,
                                value: "test"
                            }
                        ]
                    },
                    {
                        type: consts_1.TYPES.function,
                        callerId,
                        value: 'contains'
                    }
                ]
            ]);
        });
        it('path is choose(/a = /c, boolean(/d), boolean(/f))', () => {
            path = 'choose(/a = /c, boolean(/d), boolean(/f))';
            expected = [
                {
                    type: consts_1.TYPES.arguments,
                    value: [
                        [
                            {
                                callerId,
                                type: consts_1.TYPES.rootPath,
                                value: '/a'
                            },
                            {
                                callerId,
                                type: consts_1.TYPES.rootPath,
                                value: '/c'
                            },
                            {
                                callerId,
                                type: consts_1.TYPES.operator,
                                value: '='
                            }
                        ],
                        [
                            {
                                type: consts_1.TYPES.arguments,
                                value: [
                                    {
                                        callerId,
                                        type: consts_1.TYPES.rootPath,
                                        value: '/d'
                                    }
                                ]
                            },
                            {
                                callerId,
                                type: consts_1.TYPES.function,
                                value: 'boolean'
                            }
                        ],
                        [
                            {
                                type: consts_1.TYPES.arguments,
                                value: [
                                    {
                                        callerId,
                                        type: consts_1.TYPES.rootPath,
                                        value: '/f'
                                    }
                                ]
                            },
                            {
                                callerId,
                                type: consts_1.TYPES.function,
                                value: 'boolean'
                            }
                        ]
                    ]
                },
                {
                    callerId,
                    type: consts_1.TYPES.function,
                    value: 'choose'
                }
            ];
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
        });
        it('path is /*[name()="a" or name()="b"]', () => {
            const path = '/*[name()="a" or name()="b"]';
            expected = [
                {
                    type: consts_1.TYPES.rootPath,
                    value: "/*",
                    callerId: 'main'
                },
                [
                    {
                        type: consts_1.TYPES.arguments,
                        value: []
                    },
                    {
                        type: consts_1.TYPES.function,
                        value: 'name',
                        callerId: 'main'
                    },
                    {
                        type: consts_1.TYPES.string,
                        value: 'a'
                    },
                    {
                        type: consts_1.TYPES.operator,
                        value: '=',
                        callerId: 'main'
                    },
                    {
                        type: consts_1.TYPES.arguments,
                        value: []
                    },
                    {
                        type: consts_1.TYPES.function,
                        value: 'name',
                        callerId: 'main'
                    },
                    {
                        type: consts_1.TYPES.string,
                        value: 'b'
                    },
                    {
                        type: consts_1.TYPES.operator,
                        value: '=',
                        callerId: 'main'
                    },
                    {
                        type: consts_1.TYPES.operator,
                        value: 'or',
                        callerId: 'main'
                    }
                ]
            ];
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
        });
        it('path is //*[local-name()="a" and sibling::b="c"]', () => {
            const path = '//*[local-name()="a" and sibling::b="c"]';
            expected = [
                {
                    type: consts_1.TYPES.rootPath,
                    value: "//*",
                    callerId: 'main'
                },
                [
                    {
                        type: consts_1.TYPES.arguments,
                        value: []
                    },
                    {
                        type: consts_1.TYPES.function,
                        value: 'local-name',
                        callerId: 'main'
                    },
                    {
                        type: consts_1.TYPES.string,
                        value: 'a'
                    },
                    {
                        type: consts_1.TYPES.operator,
                        value: '=',
                        callerId: 'main'
                    },
                    {
                        type: consts_1.TYPES.path,
                        value: './sibling::b',
                        callerId: 'main'
                    },
                    {
                        type: consts_1.TYPES.string,
                        value: 'c'
                    },
                    {
                        type: consts_1.TYPES.operator,
                        value: '=',
                        callerId: 'main'
                    },
                    {
                        type: consts_1.TYPES.operator,
                        value: 'and',
                        callerId: 'main'
                    }
                ]
            ];
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
        });
        it('path is count(//*)', () => {
            const path = 'count(//*)';
            expected = [
                {
                    type: consts_1.TYPES.arguments,
                    value: [
                        {
                            callerId: 'main',
                            type: 'rootPath',
                            value: '//*'
                        }
                    ]
                },
                {
                    callerId: 'main',
                    type: consts_1.TYPES.function,
                    value: 'count'
                }
            ];
            expect(postfixInstance.toPostfix(path)).toEqual(expected);
        });
    });
    describe('Variables', () => {
        beforeEach(() => {
            postfixInstance = new Postfix_1.Postfix(callerId, functionsInstance);
        });
        it('path is $Var1/d/c', () => {
            path = '$Var1/d/c';
            expect(postfixInstance.toPostfix(path)).toEqual([
                {
                    callerId,
                    type: consts_1.TYPES.variablePath,
                    value: '$Var1/d/c'
                }
            ]);
        });
        it('path is $Var1[f=1]/c = $Var2[l=$Var3]/k', () => {
            path = '$Var1[f=1]/c = $Var2[l=$Var3]/k';
            expect(postfixInstance.toPostfix(path)).toEqual([
                {
                    type: consts_1.TYPES.variable,
                    value: '$Var1'
                },
                [
                    {
                        callerId,
                        type: consts_1.TYPES.path,
                        value: './f'
                    },
                    {
                        type: consts_1.TYPES.number,
                        value: 1
                    },
                    {
                        callerId,
                        type: consts_1.TYPES.operator,
                        value: '='
                    }
                ],
                {
                    callerId,
                    type: consts_1.TYPES.path,
                    value: '/c'
                },
                {
                    type: consts_1.TYPES.variable,
                    value: '$Var2'
                },
                [
                    {
                        callerId,
                        type: consts_1.TYPES.path,
                        value: './l'
                    },
                    {
                        type: consts_1.TYPES.variable,
                        value: '$Var3'
                    },
                    {
                        callerId,
                        type: consts_1.TYPES.operator,
                        value: '='
                    }
                ],
                {
                    callerId,
                    type: consts_1.TYPES.path,
                    value: '/k'
                },
                {
                    callerId,
                    type: consts_1.TYPES.operator,
                    value: '='
                }
            ]);
        });
    });
});
//# sourceMappingURL=Postfix.spec.js.map